@@ -1,8 +1,9 @@
+import datetime
 from copy import copy
 
 from pytest import fixture, raises
 from pyluach import dates, hebrewcal
-from pyluach.hebrewcal import Year, Month, holiday
+from pyluach.hebrewcal import Year, Month, holiday, festival, fast_day
 
 
 class TestYear:
@@ -60,6 +61,21 @@ class TestYear:
         with raises(ValueError):
             Year(0)
 
+    def test_year_string(self):
+        year = Year(5781)
+        assert year.year_string() == 'תשפ״א'
+        assert year.year_string(True) == 'ה׳תשפ״א'
+
+    def test_from_date(self):
+        date = dates.GregorianDate(2021, 6, 7)
+        year = Year.from_date(date)
+        assert year == Year(date.to_heb().year)
+
+    def test_from_pydate(self):
+        pydate = datetime.date(2021, 6, 7)
+        date = dates.HebrewDate.from_pydate(pydate)
+        assert Year.from_pydate(pydate) == Year(date.year)
+
 
 @fixture
 def years():
@@ -179,12 +195,34 @@ class TestMonth:
             'weekday': 7, 'hour': 3, 'minutes': 23, 'parts': 0
         }
     
+    def test_month_name(self):
+        month = Month(5781, 9)
+        assert month.month_name() == 'Kislev'
+        assert month.month_name(hebrew=True) == 'כסלו'
+        adar = Month(5781, 12)
+        assert adar.month_name() == 'Adar'
+        adar_bais = Month(5782, 13)
+        assert adar_bais.month_name() == 'Adar 2'
+
+    def test_month_string(self):
+        month = Month(5781, 3)
+        assert month.month_string() == 'סיון תשפ״א'
+        assert month.month_string(True) == 'סיון ה׳תשפ״א'
+
     def test_errors(self):
         with raises(ValueError):
             Month(-1, 1)
         with raises(ValueError):
             Month(5781, 13)
 
+    def test_from_date(self):
+        date = dates.HebrewDate(5781, 7, 10)
+        assert Month.from_date(date) == Month(date.year, date.month)
+
+    def test_from_pydate(self):
+        pydate = datetime.date(2021, 6, 7)
+        date = dates.HebrewDate.from_pydate(pydate)
+        assert Month.from_pydate(pydate) == Month(date.year, date.month)
 
 @fixture
 def months():
@@ -232,13 +270,15 @@ class TestHoliday:
                    ])
 
     def test_yomkippur(self):
-        assert holiday(dates.HebrewDate(5775, 7, 10)) == 'Yom Kippur'
+        yom_kippur = dates.HebrewDate(5775, 7, 10)
+        assert holiday(yom_kippur) == 'Yom Kippur'
+        assert holiday(yom_kippur, hebrew=True) == 'יום כיפור'
 
     def test_succos(self):
         day = dates.HebrewDate(5778, 7, 18)
-        assert holiday(day) == 'Succos'
+        assert festival(day) == 'Succos'
         day2 = dates.HebrewDate(5778, 7, 23)
-        assert holiday(day2, israel=True) is None
+        assert festival(day2, israel=True, hebrew=True) is None
 
     def test_shmini(self):
         shmini = dates.HebrewDate(5780, 7, 22)
@@ -261,7 +301,7 @@ class TestHoliday:
         purims = [dates.HebrewDate(5778, 12, 14),
                   dates.HebrewDate(5779, 13, 14)]
         for purim in purims:
-            assert holiday(purim) == 'Purim'
+            assert holiday(purim, hebrew=True) == 'פורים'
             assert holiday(purim + 1) == 'Shushan Purim'
         assert holiday(dates.HebrewDate(5779, 12, 14)) == 'Purim Katan'
 
@@ -282,7 +322,9 @@ class TestHoliday:
         assert holiday(ps + 1) is None
 
     def test_lagbaomer(self):
-        assert holiday(dates.GregorianDate(2018, 5, 3)) == "Lag Ba'omer"
+        lag_baomer = dates.GregorianDate(2018, 5, 3)
+        assert festival(lag_baomer) == "Lag Ba'omer"
+        assert festival(lag_baomer, hebrew=True) == 'ל״ג בעומר'
 
     def test_shavuos(self):
         shavuos = dates.HebrewDate(5778, 3, 6)
@@ -296,12 +338,14 @@ class TestHoliday:
 class TestFasts:
 
     def test_gedalia(self):
-        assert holiday(dates.HebrewDate(5779, 7, 3)) == 'Tzom Gedalia'
+        assert fast_day(dates.HebrewDate(5779, 7, 3)) == 'Tzom Gedalia'
         assert holiday(dates.HebrewDate(5778, 7, 3)) is None
-        assert holiday(dates.HebrewDate(5778, 7, 4)) == 'Tzom Gedalia'
+        assert holiday(dates.HebrewDate(5778, 7, 4), hebrew=True) == 'צום גדליה'
 
     def test_asara(self):
-        assert holiday(dates.GregorianDate(2018, 12, 18)) == '10 of Teves'
+        ten_of_teves = dates.GregorianDate(2018, 12, 18)
+        assert holiday(ten_of_teves) == '10 of Teves'
+        assert fast_day(ten_of_teves, hebrew=True) == 'י׳ בטבת'
 
     def test_esther(self):
         fasts = [
